@@ -16,6 +16,7 @@
 from __future__ import division, absolute_import
 
 import functools
+import platform
 import six
 import struct
 import weakref
@@ -25,9 +26,15 @@ try:
 except ImportError:
     from xcffib.ffi_build import ffi
 
-lib = ffi.dlopen('libxcb.so.1')
+soname = "libxcb.so.1"
+if platform.system() == "Darwin":
+    soname = "libxcb.dylib"
+elif platform.system() == "Windows":
+    soname = "libxcb.dll"
+lib = ffi.dlopen(soname)
 
-__xcb_proto_version__ = "1.11"
+__xcb_proto_version__ = "1.13"
+__version__ = 'placeholder'
 
 X_PROTOCOL = lib.X_PROTOCOL
 X_PROTOCOL_REVISION = lib.X_PROTOCOL_REVISION
@@ -240,7 +247,7 @@ def _add_ext(key, value, events, errors):
 
 class ExtensionKey(object):
 
-    """ This definitely isn't needed, but we keep it around for compatibilty
+    """ This definitely isn't needed, but we keep it around for compatibility
     with xpyb.
     """
 
@@ -290,6 +297,12 @@ class Protobj(object):
             self.bufsize = unpacker.known_max
 
 
+class Buffer(Protobj):
+    def __init__(self, unpacker):
+        Protobj.__init__(self, unpacker)
+        self.buf = unpacker.buf
+
+
 class Struct(Protobj):
     pass
 
@@ -320,6 +333,9 @@ class Cookie(object):
             "Request is not void and checked")
         self.conn.request_check(self.sequence)
 
+    def discard_reply(self):
+        return self.conn.discard_reply(self.sequence)
+
 
 class VoidCookie(Cookie):
 
@@ -408,6 +424,8 @@ class List(Protobj):
 
         self.bufsize = unpacker.offset - old
 
+        self.raw = bytes(unpacker.buf[old:old + self.bufsize])
+
         assert count is None or count == len(self.list)
 
     def __str__(self):
@@ -439,10 +457,19 @@ class List(Protobj):
 
     def to_atoms(self):
         """ A helper for converting a List of chars to an array of atoms """
-        return struct.unpack("=" + "I" * (len(self) // 4), b''.join(self))
+        return struct.unpack("<" + "I" * (len(self) // 4), b''.join(self))
 
     def buf(self):
-        return b''.join(self.list)
+        return self.raw
+
+    @classmethod
+    def synthetic(cls, list=None):
+        if list is None:
+            list = []
+
+        self = cls.__new__(cls)
+        self.list = list[:]
+        return self
 
 
 class OffsetMap(object):
@@ -633,7 +660,10 @@ class Connection(object):
 
         reply = ffi.cast("xcb_generic_reply_t *", data)
 
-        # why is this 32 and not sizeof(xcb_generic_reply_t) == 8?
+        # this is 32 and not `sizeof(xcb_generic_reply_t) == 8` because,
+        # according to the X11 protocol specs: "Every reply consists of 32 bytes
+        # followed by zero or more additional bytes of data, as specified in the
+        # length field."
         return CffiUnpacker(data, known_max=32 + reply.length * 4)
 
     @ensure_connected
@@ -662,6 +692,10 @@ class Connection(object):
     def send_request(self, flags, xcb_parts, xcb_req):
         return lib.xcb_send_request(self._conn, flags, xcb_parts, xcb_req)
 
+    @ensure_connected
+    def discard_reply(self, sequence):
+        return lib.xcb_discard_reply(self._conn, sequence)
+
 
 # More backwards compatibility
 connect = Connection
@@ -742,13 +776,13 @@ def pack_list(from_, pack_type):
 
     if isinstance(pack_type, six.string_types):
         return struct.pack("=" + pack_type * len(from_), *from_)
-    elif hasattr(pack_type, "synthetic"):
-        return pack_type.synthetic(*from_).pack()
     else:
         buf = six.BytesIO()
         for item in from_:
-            # If we can't pack it, you'd better have packed it yourself...
-            if isinstance(item, Struct):
+            # If we can't pack it, you'd better have packed it yourself. But
+            # let's not confuse things which aren't our Probobjs for packable
+            # things.
+            if isinstance(item, Protobj) and hasattr(item, "pack"):
                 buf.write(item.pack())
             else:
                 buf.write(item)
